//
//  UICountGradientLabel.m
//  NBChart
//
//  Created by Jax on 2016/11/22.
//  Copyright © 2016年 Jax. All rights reserved.
//

#define kUICountRate 3.0

#import "UICountGradientLabel.h"

#pragma mark -

@protocol UICountLabel <NSObject>

- (CGFloat)update:(CGFloat)t;

@end

@interface UICountEaseInOut : NSObject <UICountLabel>
@end

@implementation UICountEaseInOut

- (CGFloat)update:(CGFloat)t {
    int sign = 1;
    int r = (int)kUICountRate;
    if (r % 2 == 0) sign = -1;
    t *= 2;
    if (t < 1) {
        return 0.5f * powf(t,  kUICountRate);
    } else {
        return sign * 0.5 * (powf(t-2, kUICountRate) + sign * 2);
    }
}

@end

@interface UICountEaseIn : NSObject <UICountLabel>
@end

@implementation UICountEaseIn

- (CGFloat)update:(CGFloat)t {
    return pow(t, kUICountRate);
}

@end

@interface UICountEaseOut : NSObject <UICountLabel>
@end

@implementation UICountEaseOut

- (CGFloat)update:(CGFloat)t {
    return 1.0 - powf((1.0 - t), kUICountRate);
}

@end

@interface UICountLinear : NSObject <UICountLabel>
@end

@implementation UICountLinear

- (CGFloat)update:(CGFloat)t {
    return t;
}

@end

#pragma mark -

@interface UICountGradientLabel ()

@property (nonatomic, assign) CGFloat startingValue;
@property (nonatomic, assign) CGFloat destinationValue;
@property (nonatomic, assign) NSTimeInterval totalTime;
@property (nonatomic, assign) NSTimeInterval progress;
@property (nonatomic, assign) NSTimeInterval lastUpdate;

@property (nonatomic, strong) id<UICountLabel> countType;

@property (nonatomic, strong) CADisplayLink *timer;

@end

@implementation UICountGradientLabel

- (void)countGradientFrom:(CGFloat)startValue to:(CGFloat)endValue {
    if (self.animationDuration == 0.0f) {
        self.animationDuration = 2.0f;
    }
    [self countGradientFrom:startValue to:endValue duration:self.animationDuration];
}

- (void)countGradientFrom:(CGFloat)startValue to:(CGFloat)endValue duration:(NSTimeInterval)duration {
    self.startingValue = startValue;
    self.destinationValue = endValue;
    
    [self.timer invalidate];
    self.timer = nil;

    if (duration == 0.0) {
        [self setLabeValue:endValue];
        return ;
    }

    self.progress = 0.0f;
    self.lastUpdate = [NSDate timeIntervalSinceReferenceDate];
    self.totalTime = duration;
    
    if (!self.format) {
        self.format = @"%0.1f";
    }
    
    switch (self.countGradientType) {
        case UICountGradientTypeEaseInOut:
            self.countType = [[UICountEaseInOut alloc] init];
            break;
        case UICountGradientTypeEaseIn:
            self.countType = [[UICountEaseIn alloc] init];
            break;
        case UICountGradientTypeEaseOut:
            self.countType = [[UICountEaseOut alloc] init];
            break;
        case UICountGradientTypeLinear:
            self.countType = [[UICountLinear alloc] init];
            break;
    }
    
    CADisplayLink *timer = [CADisplayLink displayLinkWithTarget:self selector:@selector(updateValue:)];
    timer.preferredFramesPerSecond = 60;
    [timer addToRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    [timer addToRunLoop:[NSRunLoop mainRunLoop] forMode:UITrackingRunLoopMode];
    self.timer = timer;
    
}

- (void)countFromCurrentValueTo:(CGFloat)endValue {
    [self countGradientFrom:[self currentValue] to:endValue];
}

- (void)countFromCurrentValueTo:(CGFloat)endValue duration:(NSTimeInterval)duration {
    [self countGradientFrom:[self currentValue] to:endValue duration:duration];
}

- (void)countFromZeroTo:(CGFloat)endValue {
    [self countGradientFrom:0.0f to:endValue];
}

- (void)countFromZeroTo:(CGFloat)endValue duration:(NSTimeInterval)duration {
    [self countGradientFrom:0.0f to:endValue duration:duration];
}

- (void)setFormat:(NSString *)format {
    _format = format;
}

- (void)updateValue:(NSTimer *)timer {
    
    NSTimeInterval now = [NSDate timeIntervalSinceReferenceDate];
    self.progress += now - self.lastUpdate;
    self.lastUpdate = now;
    
    if (self.progress >= self.totalTime) {
        [self.timer invalidate];
        self.timer = nil;
        self.progress = self.totalTime;
    }
    [self setLabeValue:[self currentValue]];
    if (self.progress == self.totalTime) {
        [self runCompleteBlock];
    }
}

- (void)runCompleteBlock {
    if (self.completeBlock) {
        self.completeBlock();
        self.completeBlock = nil;
    }
}

- (void)setLabeValue:(CGFloat)value {
    if (self.attributedFormatBlock) {
        self.attributedText = self.attributedFormatBlock(value);
    } else if (self.formatBlock) {
        self.text = self.formatBlock(value);
    } else {
        if ([self.format rangeOfString:@"%(.*)d" options:NSRegularExpressionSearch].location != NSNotFound || [self.format rangeOfString:@"%(.*)i" options:NSRegularExpressionSearch].location != NSNotFound) {
            self.text = [NSString stringWithFormat:self.format,(int)value];
        } else {
            self.text = [NSString stringWithFormat:self.format, value];
        }
    }
}

- (CGFloat)currentValue {
    if (self.progress >= self.totalTime) {
        return self.destinationValue;
    }
    
    CGFloat percent = self.progress / self.totalTime;
    CGFloat updateValue = [self.countType update:percent];
    return self.startingValue + (updateValue * (self.destinationValue - self.startingValue));
}

@end
