//
//  YBPView.m
//  仪表盘
//
//  Created by Jax on 2016/12/13.
//  Copyright © 2016年 onlyLoveYou. All rights reserved.
//

#import "YBPView.h"
#import "UICountGradientLabel.h"

#define kScreenWidth  [UIScreen mainScreen].bounds.size.width
#define kScreenHeight [UIScreen mainScreen].bounds.size.height

#define kRGBA(r, g, b, a) [UIColor colorWithRed:(r) / 255.0 green:(g) / 255.0 blue:(b) / 255.0 alpha:(a) / 255.0]
#define kRGB(r, g, b)     kRGBA(r, g, b, 255)

#define DEGREES_TO_RADIANS(angle) ((angle) / 180.0 * M_PI)
#define RADIANS_TO_DEGREES(radians)  ((180 * radians) / M_PI)

#define kDialLineWidth 15
#define kDialRadius 120


static NSInteger imageViewCenterIndex = 0;

@interface YBPView ()

@property (nonatomic, assign) NSInteger startCount;
@property (nonatomic, assign) NSInteger scale;
@property (nonatomic, assign) NSInteger toValue;

@property (nonatomic, strong) UIImageView *imageView;
@property (nonatomic, strong) NSArray *centers;
@property (nonatomic, strong) CADisplayLink *displayLink;

@property (nonatomic, strong) CAShapeLayer *shapeLayer3;
@property (nonatomic, strong) CAShapeLayer *shapeLayer4;

@property (nonatomic, assign) CGPoint preContactPoint;


@end

@implementation YBPView

- (CAShapeLayer *)layerWithLineWidth:(CGFloat)lineWidth strokeColor:(UIColor *)strokeColor fillColor:(UIColor *)fillColor {
    CAShapeLayer *shapeLayer = [CAShapeLayer layer];
    shapeLayer.lineWidth = lineWidth;
    shapeLayer.strokeColor = strokeColor.CGColor;
    shapeLayer.fillColor = fillColor.CGColor;
    return shapeLayer;
}

- (instancetype)initWithFrame:(CGRect)frame startCount:(NSInteger)startCount  scale:(NSInteger)scale toValue:(NSInteger)toValue {
    if (self = [super init]) {
        
        self.frame = frame;
        self.startCount = startCount;
        self.scale = scale;
        self.toValue = toValue;
        
        CGPoint center = CGPointMake(frame.size.width * 0.5, frame.size.height * 0.5);
        
        ///< 最外面的圆
        
        UIBezierPath *outPath = [UIBezierPath bezierPath];
        [outPath addArcWithCenter:center
                           radius:kDialRadius + 20
                       startAngle:DEGREES_TO_RADIANS(145)
                         endAngle:DEGREES_TO_RADIANS(35)
                        clockwise:YES];
        
        self.shapeLayer3 = [self layerWithLineWidth:2.0f
                                        strokeColor:kRGB(102, 111, 117)
                                          fillColor:kRGB(36, 49, 58)];
        self.shapeLayer3.path = outPath.CGPath;
        [self.layer addSublayer:self.shapeLayer3];
        
        self.centers = [NSArray arrayWithArray:[self imageViewCenters]];
        UIImageView *imageView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"走针"]];
        imageView.frame = CGRectMake(0, 0, 15, 15);
        imageView.center = [self.centers.firstObject CGPointValue];
        self.preContactPoint = imageView.center;
        [self addSubview:imageView];
        self.imageView = imageView;
        
        ///< 定时帧控制走针
        
        if (self.displayLink) [self.displayLink invalidate];
        CADisplayLink *displayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(frameImageView:)];
        displayLink.preferredFramesPerSecond = 50;
        [displayLink addToRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
        [displayLink addToRunLoop:[NSRunLoop mainRunLoop] forMode:UITrackingRunLoopMode];
        self.displayLink = displayLink;
        
        ///< 中间的圆
        
        UIBezierPath *bePath = [UIBezierPath bezierPath];
        [bePath addArcWithCenter:center
                          radius:kDialRadius
                      startAngle:DEGREES_TO_RADIANS(150)
                        endAngle:DEGREES_TO_RADIANS(30)
                       clockwise:YES];
        
        CAShapeLayer *shapeLayer = [self layerWithLineWidth:kDialLineWidth
                                                strokeColor:kRGB(102, 111, 117)
                                                  fillColor:kRGB(36, 49, 58)];
        shapeLayer.path = bePath.CGPath;
        [self.layer addSublayer:shapeLayer];
        
        ///< ------------------------------10个大刻度 20个小刻度
        
        CGFloat perAngle = 240 / 20.0;
        CGFloat perAngle2 = DEGREES_TO_RADIANS(240) / 20.0;
        
        ///< 最里面的圆
        CGFloat lineWidth = 0.5;
        for (int i = 0; i < 20; i ++) {
            
            NSInteger startAngel = 150 + perAngle * i;
            NSInteger endAngel   = startAngel + perAngle;
            if (endAngel <= 270 - perAngle) {
                lineWidth += 0.1;
            } else if (endAngel > 270 + 2 * perAngle) {
                lineWidth -= 0.1;
            }
            if (endAngel > 360) endAngel -= 360;
            
            UIBezierPath *bePath2 = [UIBezierPath bezierPath];
            [bePath2 addArcWithCenter:center
                               radius:kDialRadius - 50
                           startAngle:DEGREES_TO_RADIANS(startAngel)
                             endAngle:DEGREES_TO_RADIANS(endAngel)
                            clockwise:YES];
            CAShapeLayer *shapeLayer2 = [self layerWithLineWidth:lineWidth
                                                     strokeColor:kRGB(248, 108, 2)
                                                       fillColor:kRGB(36, 49, 58)];
            shapeLayer2.path = bePath2.CGPath;
            [self.layer addSublayer:shapeLayer2];
        }
        
        ///< 画刻度
        
        for (int i = 0; i <= 20; i ++) {
            
            CGFloat startAngel = (DEGREES_TO_RADIANS(150) + perAngle2 * i);
            CGFloat endAngel   = startAngel + perAngle2 / 8.0;
            
            if (endAngel > 2 * M_PI) endAngel -= 2 * M_PI;
            
            UIBezierPath *barPath = [UIBezierPath bezierPath];
            
            CAShapeLayer *barLayer = [CAShapeLayer layer];
            if (i % 2 == 0) {
                [barPath addArcWithCenter:center
                                   radius:kDialRadius - 3
                               startAngle:startAngel
                                 endAngle:endAngel
                                clockwise:YES];
                barLayer.strokeColor = kRGB(255, 255, 255).CGColor;
                barLayer.lineWidth = kDialLineWidth + 6;
            } else {
                [barPath addArcWithCenter:center
                                   radius:kDialRadius + 3
                               startAngle:startAngel
                                 endAngle:endAngel
                                clockwise:YES];
                barLayer.strokeColor = kRGB(248, 108, 2).CGColor;
                barLayer.lineWidth = kDialLineWidth - 6;
            }
            
            barLayer.path = barPath.CGPath;
            
            [self.layer addSublayer:barLayer];
            
        }
        
        ///< 刻度值 scale = 40
        
        NSInteger count = -scale;
        CGFloat textR = kDialRadius - 30;
        NSInteger startDu = 126;
        ///< (textX, textY) 刻度值label的center
        CGFloat textX = 0;
        CGFloat textY = 0;
        
        for (int i = 0; i <= 10; i ++) {
            startDu += 24;
            count += scale;
            if (startDu <= 180) {
                CGFloat startDu1 = 180 - startDu;
                CGFloat x = textR * cos(DEGREES_TO_RADIANS(startDu1));
                CGFloat y = textR * sin(DEGREES_TO_RADIANS(startDu1));
                textX = center.x - x;
                textY = center.y + y;
            } else if (startDu <= 270) {
                CGFloat startDu2 = 270 - startDu;
                CGFloat x = textR * sin(DEGREES_TO_RADIANS(startDu2));
                CGFloat y = textR * cos(DEGREES_TO_RADIANS(startDu2));
                textX = center.x - x;
                textY = center.y - y;
            } else if (startDu <= 360) {
                CGFloat startDu3 = 360 - startDu;
                CGFloat x = textR * cos(DEGREES_TO_RADIANS(startDu3));
                CGFloat y = textR * sin(DEGREES_TO_RADIANS(startDu3));
                textX = center.x + x;
                textY = center.y - y;
            } else {
                CGFloat startDu3 = startDu - 360;
                CGFloat x = textR * cos(DEGREES_TO_RADIANS(startDu3));
                CGFloat y = textR * sin(DEGREES_TO_RADIANS(startDu3));
                textX = center.x + x;
                textY = center.y + y;
            }
            
            UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 25, 15)];
            label.textAlignment = NSTextAlignmentCenter;
            label.center = CGPointMake(textX, textY);
            label.textColor = kRGB(204, 207, 209);
            label.text = [NSString stringWithFormat:@"%ld", (long)count];
            label.font = [UIFont systemFontOfSize:11];
            [self addSubview:label];
            
        }
        
        UILabel *label = [[UILabel alloc] init];
        label.text = @"行驶里程";
        label.textAlignment = NSTextAlignmentCenter;
        label.frame = CGRectMake(0, 0, 100, 25);
        label.center = CGPointMake(center.x, center.y + 40);
        label.textColor = kRGB(204, 207, 209);
        label.font = [UIFont systemFontOfSize:13];
        [self addSubview:label];
        
        UICountGradientLabel *label2 = [[UICountGradientLabel alloc] init];
        label2.format = @"%dKM";
        label2.textAlignment = NSTextAlignmentCenter;
        label2.frame = CGRectMake((kScreenWidth - 90) * 0.5, CGRectGetMaxY(label.frame), 90, 25);
        label2.textColor = kRGB(204, 207, 209);
        label2.backgroundColor = kRGB(84, 104, 111);
        label2.font = [UIFont systemFontOfSize:16];
        [self addSubview:label2];
        [label2 countFromZeroTo:8000];
        
        UILabel *label3 = [[UILabel alloc] init];
        label3.text = @"订单";
        label3.textAlignment = NSTextAlignmentCenter;
        label3.frame = CGRectMake(0, 0, 100, 20);
        label3.center = CGPointMake(center.x, center.y - 45);
        label3.textColor = kRGB(204, 207, 209);
        label3.font = [UIFont systemFontOfSize:13];
        [self addSubview:label3];
        
        UICountGradientLabel *label4 = [[UICountGradientLabel alloc] init];
        label4.format = @"%d";
        label4.textAlignment = NSTextAlignmentCenter;
        label4.frame = CGRectMake((kScreenWidth - 90) * 0.5, CGRectGetMaxY(label3.frame), 90, 50);
        label4.textColor = kRGB(165, 133, 1);
        label4.font = [UIFont boldSystemFontOfSize:40];
        label4.layer.shadowColor = kRGBA(165, 133, 1, 150).CGColor;
        label4.shadowOffset = CGSizeMake(0, -10);
        [self addSubview:label4];
        [label4 countFromZeroTo:self.toValue];

    }
    return self;
}

- (void)frameImageView:(NSTimer *)timer {
    imageViewCenterIndex ++;
    if (imageViewCenterIndex == self.centers.count - 1) {
        imageViewCenterIndex = 0;
        if (self.displayLink) {
            [self.displayLink invalidate];
        }
    } else {
       CGPoint currentPoint = [self.centers[imageViewCenterIndex] CGPointValue];
       self.imageView.center = currentPoint;
        
        ///< 计算切线角度
        CGFloat qieJiao = atan((currentPoint.y - self.preContactPoint.y) / (currentPoint.x - self.preContactPoint.x));
        self.preContactPoint = currentPoint;
        
        self.imageView.transform = CGAffineTransformIdentity;
        self.imageView.transform = CGAffineTransformMakeRotation(RADIANS_TO_DEGREES(-135.5) +qieJiao);
        
        if (self.shapeLayer4) {
            [self.shapeLayer4 removeFromSuperlayer];
        }
        CGPoint center = CGPointMake(self.frame.size.width * 0.5, self.frame.size.height * 0.5);
        UIBezierPath *outPath = [UIBezierPath bezierPath];
        [outPath addArcWithCenter:center
                           radius:kDialRadius + 20
                       startAngle:DEGREES_TO_RADIANS(145)
                         endAngle:DEGREES_TO_RADIANS(35) clockwise:YES];
        
        CAShapeLayer *shapeLayer4 = [CAShapeLayer layer];
        shapeLayer4.lineWidth = 2;
        shapeLayer4.strokeColor = [UIColor whiteColor].CGColor;
        shapeLayer4.fillColor = kRGB(36, 49, 58).CGColor;
        shapeLayer4.lineCap = kCALineCapRound;
        shapeLayer4.path = outPath.CGPath;
        [self.layer addSublayer:shapeLayer4];
        [self.layer insertSublayer:shapeLayer4 above:self.shapeLayer3];
        self.shapeLayer4 = shapeLayer4;
        shapeLayer4.strokeEnd = (self.toValue / (self.startCount + self.scale * 10.0)) * (imageViewCenterIndex / 100.0);
    }
}

- (NSArray *)imageViewCenters {
    
    CGFloat totalDu = (self.toValue / (self.startCount + self.scale * 10.0)) * 250;
    ///<取圆上的100个点
    CGFloat perDu = totalDu / 100;
    
    CGPoint center = CGPointMake(self.frame.size.width * 0.5, self.frame.size.height * 0.5);
    
    NSMutableArray *centers = [NSMutableArray array];
    
    CGFloat textR = kDialRadius + 20;
    CGFloat startDu = 145 - perDu;
    
    CGFloat textX = 0;
    CGFloat textY = 0;
    
    for (int i = 0; i < 100; i ++) {
        startDu += perDu;
        if (startDu <= 180) {
            CGFloat startDu1 = 180 - startDu;
            CGFloat x = textR * cos(DEGREES_TO_RADIANS(startDu1));
            CGFloat y = textR * sin(DEGREES_TO_RADIANS(startDu1));
            textX = center.x - x;
            textY = center.y + y;
        } else if (startDu <= 270) {
            CGFloat startDu2 = 270 - startDu;
            CGFloat x = textR * sin(DEGREES_TO_RADIANS(startDu2));
            CGFloat y = textR * cos(DEGREES_TO_RADIANS(startDu2));
            textX = center.x - x;
            textY = center.y - y;
        } else if (startDu <= 360) {
            CGFloat startDu3 = 360 - startDu;
            CGFloat x = textR * cos(DEGREES_TO_RADIANS(startDu3));
            CGFloat y = textR * sin(DEGREES_TO_RADIANS(startDu3));
            textX = center.x + x;
            textY = center.y - y;
        } else {
            CGFloat startDu3 = startDu - 360;
            CGFloat x = textR * cos(DEGREES_TO_RADIANS(startDu3));
            CGFloat y = textR * sin(DEGREES_TO_RADIANS(startDu3));
            textX = center.x + x;
            textY = center.y + y;
        }
        
        [centers addObject:[NSValue valueWithCGPoint:CGPointMake(textX, textY)]];
        
    }
    return centers.copy;

}


@end
